library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity adrcnt is
    generic(
        PRM: integer;
	      SIZE_RAM : integer 
	  );
    port( 
        clk       : in std_logic;
        rst       : in std_logic;

        start     : in std_logic;
        enable    : in std_logic;
        stop      : out std_logic;
       
        adr_start : in std_logic_vector(SIZE_RAM - 1 downto 0); -- index start address of bitstream
        adr_end   : in std_logic_vector(SIZE_RAM - 1 downto 0); -- index end address of bitstream

        adr_bit   : out std_logic_vector(SIZE_RAM - 1 downto 0); -- address into memory
        adr_valid : out std_logic
    ); 
end adrcnt;


architecture behav of adrcnt is

signal adr_valid_sig : std_logic;
signal cnt_reg : std_logic_vector(SIZE_RAM - 1 downto 0);
signal stop_sig : std_logic;

begin
    stop_sig <= '0' when (cnt_reg < adr_end) else '1';    --(cnt_reg >= adr_start) and (cnt_reg < adr_end) else '1';

    process(clk, rst)
    begin
        if (rst = '1') then
            cnt_reg <= (others => '0');
            adr_valid_sig <= '0';
            --stop <= '0';
        else
            if rising_edge(clk) then
                if (start = '1') then
    	              cnt_reg <= adr_start;
    	              --stop <= '0';
    	              adr_valid_sig <= '1';
                elsif  (enable = '1') and  (stop_sig = '0') then
                    cnt_reg <= cnt_reg + 1;
                    adr_valid_sig <= '1';
                elsif  (enable = '1') and  (stop_sig = '1') then
                    --stop <= '1';
                    adr_valid_sig <= '0';
                else 
                    adr_valid_sig <= '0';
                end if;
            end if;
        end if;
    end process;

    stop <= stop_sig;
    adr_bit <= cnt_reg;
    adr_valid <= adr_valid_sig;

end behav;

