library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity flash_controller is
generic(
	   SIZE_RAM : integer 
	  );
port (
	oe_n : out std_logic;
	we_n : out std_logic;
	ce_n : out std_logic;
	addr : out std_logic_vector(SIZE_RAM-1 downto 0);
	io : in std_logic_vector(7 downto 0);
	
	address : in std_logic_vector(SIZE_RAM-1 downto 0);	
	rd : in std_logic;
	reset : in std_logic;
	data : out std_logic_vector(7 downto 0);
	data_valid : out std_logic;
	clk : in std_logic	-- MAX 10 MHz
--	chipscope_flash : out std_logic_vector(2 downto 0)
);

end flash_controller;

architecture rtl of flash_controller is
type states is (
	INIT,
	IDLE,	-- wait for read or write pulse
	READ_DATA,
	READ_DATA_VALID
);


signal state: states;
signal nextstate: states;

begin
  
    state_current : process(clk, reset)
	  begin
        if (reset='1') then
            state <= INIT;
		  	elsif rising_edge(clk) then
            state <= nextstate;
			  end if;
    end process;
	 
	 
	 
	 
	 process(state, rd)
	 begin	
		  oe_n <= '0';
	     ce_n <= '0';
		  we_n <= '0';
        data_valid <= '0';	  
--	     chipscope_flash <= "000";
		  
		  case state is
		      when INIT => 
--				    chipscope_flash <= "001";
                nextstate <= IDLE;
               
            -- idle stav, ek se na nastaven ten i zpisu	
				when IDLE => 
--				    chipscope_flash <= "010";
				    oe_n <= '1';
			       ce_n <= '1';
		          if (rd = '1') then
						  nextstate <= READ_DATA;
					 else
					     nextstate <= IDLE;
					 end if;
			
		 		when READ_DATA =>
--				    chipscope_flash <= "011";
                we_n <= '1';					 
					 nextstate <= READ_DATA_VALID;
					 
			   when READ_DATA_VALID => 
--				    chipscope_flash <= "100";
					 data_valid <= '1';
                nextstate <= IDLE;	
					 
				when others => nextstate <= IDLE;
        end case;
    end process;
  
    
    addr <= address;
    data <= io;

end rtl;


