library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity fsm is
    port( 
        clk         : in std_logic;
        rst         : in std_logic;

        start_reconfig : out std_logic;                  -- impuls pri startu rekonfigurace PRM
        end_reconfig : out std_logic;                    -- impuls pri konci rekonfigurace PRM
        err         : in std_logic;
        empty_fifo  : in std_logic;
        read_fifo   : out std_logic;
      
        clear_input_reg : out std_logic;
      
        enable_safety_w : out std_logic;
        sync        : in std_logic;
      
        refresh_hard : out std_logic;
        clear_hard  : out std_logic;
        enable_hard : out std_logic;
      
        start_cnt   : out std_logic;
        enable_cnt  : out std_logic;
        stop_cnt    : in std_logic;
      
        done_rr     : in std_logic;      
        enable_rr   : out std_logic;

        parity_start : out std_logic;
        parity_ok   : in std_logic;
      
        data_valid  : in std_logic;
        data        : in std_logic_vector(7 downto 0)
   ); 
end fsm;


architecture behav of fsm is
type states is (
    INIT,
    IDLE,
    WAIT_FOR_ERROR,
    FOUND_ERROR,
    READ_FIRST_HEADER_WAIT,
    READ_HEADER,	
    READ_HEADER_WAIT,
    BYTE_COUNT1,
    BYTE_COUNT1_WAIT,
    BYTE_COUNT2,
    WRITE_HEADER,
    READ_FRAME_INIT,
    READ_FRAME,
    READ_FRAME_WAIT,
    ECC,
    WRITE_FRAME,
    READ_TAIL,
    READ_TAIL_WAIT,
    WRITE_TAIL,
    HARD_REFRESH,
    SAFETY_WINDOW,
    HARD_INIT,
    SAFETY_WINDOW2
);

signal state : states := INIT;
signal nextstate : states := INIT;
signal read_done_78 : std_logic;
signal read_done_164 : std_logic;

signal frame_byte_count : std_logic_vector(13 downto 0);

signal byte_cnt : std_logic_vector(7 downto 0);
signal frame_cnt : std_logic_vector(15 downto 0);                                --pozor na maximalni pocet slov ve vsech framech
signal byte_cnt_clear, byte_cnt_enable, frame_cnt_clear, frame_cnt_enable : std_logic;
signal enable_byte1, enable_byte2 : std_logic;
signal all_frames_read : std_logic;
signal error_reg, error_reg_clear, error_reg_enable : std_logic;

begin

    read_done_78 <= '0' when (byte_cnt < 106) else '1';
    read_done_164 <= '0' when (byte_cnt < 164) else '1';
    all_frames_read <= '0' when (frame_cnt < frame_byte_count) else '1';
   
    state_current : process(clk,rst)
	  begin
        if (rst='1') then
            state <= INIT;
        elsif rising_edge(clk) then
            state <= nextstate;
		
			  end if;
    end process;
     

	
	
    state_machine : process(state, err, data_valid, read_done_78, read_done_164, parity_ok, all_frames_read, stop_cnt, done_rr, empty_fifo, sync, error_reg)
	  begin
        read_fifo <= '0';
        enable_rr <= '0';
        enable_safety_w <= '0';
        clear_input_reg <= '0';
        refresh_hard <= '0';
        clear_hard <= '0';
        enable_hard <= '1';
        start_cnt <= '0';
        enable_cnt <= '0';
        parity_start <= '0';
        byte_cnt_clear <= '0';
        frame_cnt_clear <= '0';
        byte_cnt_enable <= '0';
        frame_cnt_enable <= '0';
        error_reg_clear <= '0';
        error_reg_enable <= '0';
        enable_byte1 <= '0';
        enable_byte2 <= '0';	
        start_reconfig <= '0';	  
        end_reconfig <= '0';    
        nextstate <= INIT;


        case state is
		        
        when INIT =>
            byte_cnt_clear <= '1';
            frame_cnt_clear <= '1';
            error_reg_clear <= '1';
            enable_hard <= '0';
            nextstate <= IDLE;
               
        when IDLE =>
            enable_rr <= '1';
            enable_hard <= '0';
            nextstate <= WAIT_FOR_ERROR;
					 
        when WAIT_FOR_ERROR =>
            if (err = '1') then
                nextstate <= FOUND_ERROR;
            elsif (done_rr = '1') and (error_reg = '1') then
                nextstate <= HARD_REFRESH;
            else
                nextstate <= IDLE;
            end if;
                
        when FOUND_ERROR => 
            error_reg_enable <= '1';
            start_cnt <= '1';
            byte_cnt_clear <= '1';
            start_reconfig <= '1';	     
            nextstate <= READ_FIRST_HEADER_WAIT;            

        when READ_FIRST_HEADER_WAIT =>
            if (data_valid = '1') then
                nextstate <= READ_HEADER;
            else
                nextstate <= READ_FIRST_HEADER_WAIT;
            end if;
                
        when READ_HEADER =>
            byte_cnt_enable <= '1';
            enable_cnt <= '1';
            nextstate <= READ_HEADER_WAIT;    
                      
        when READ_HEADER_WAIT =>
            if (read_done_78 = '1') and (data_valid = '1') then
                nextstate <= BYTE_COUNT1;
            elsif (read_done_78 = '0') and (data_valid = '1') then
                nextstate <= READ_HEADER;
            else 
                nextstate <= READ_HEADER_WAIT;
            end if;
               
        when BYTE_COUNT1 =>
            enable_cnt <= '1';
            enable_byte1 <= '1';
            nextstate <= BYTE_COUNT1_WAIT;
					 
        when BYTE_COUNT1_WAIT =>	
            if (data_valid = '1') then
                nextstate <= BYTE_COUNT2;
            else
                nextstate <= BYTE_COUNT1_WAIT;
            end if; 
            
        when BYTE_COUNT2 =>
            enable_byte2 <= '1';
            frame_cnt_clear <= '1';
            nextstate <= WRITE_HEADER;
                  		
        when WRITE_HEADER =>
            read_fifo <= '1';			 
            if(empty_fifo = '1') then
                nextstate <= READ_FRAME_INIT;
            else 
                nextstate <= WRITE_HEADER;
            end if;
					 
        when READ_FRAME_INIT =>
            parity_start <= '1';           
            byte_cnt_clear <= '1';
            nextstate <= READ_FRAME;
                
        when READ_FRAME =>
            enable_cnt <= '1';
            byte_cnt_enable <= '1';
            frame_cnt_enable <= '1';
            nextstate <= READ_FRAME_WAIT;
                          
        when READ_FRAME_WAIT =>
            if (read_done_164 = '1') and (data_valid = '1') then
                nextstate <= ECC;
            elsif(read_done_164 = '0') and (data_valid = '1') then
                nextstate <= READ_FRAME;
            else
                nextstate <= READ_FRAME_WAIT;
            end if;
            
        when ECC =>
            if (parity_ok = '1') then
                nextstate <= WRITE_FRAME;
            else
                --nextstate <= ECC;                          -- TO DO
                nextstate <= WRITE_FRAME;                    -- prozatim
            end if;
            
        when WRITE_FRAME =>
            read_fifo <= '1';
            if(all_frames_read='0') and (empty_fifo = '1') then 
                nextstate <= READ_FRAME_INIT;
            elsif(all_frames_read='1') and (empty_fifo = '1') then
                nextstate <= READ_TAIL;
            else
                nextstate <= WRITE_FRAME;
            end if;							  
                
        when READ_TAIL =>
            enable_cnt <= '1';
            nextstate <= READ_TAIL_WAIT;
                       
        when READ_TAIL_WAIT =>
            if (stop_cnt = '1') and (data_valid = '1') then
                nextstate <= WRITE_TAIL;
            elsif (stop_cnt = '0') and (data_valid = '1') then
                nextstate <= READ_TAIL;
            else 
                nextstate <= READ_TAIL_WAIT;
            end if;
            
        when WRITE_TAIL =>
            read_fifo <= '1';
            if(done_rr = '0') and (empty_fifo = '1') then 
                nextstate <= IDLE;
            elsif(done_rr = '1') and (empty_fifo = '1') then 
                nextstate <= HARD_REFRESH;
            else
                nextstate <= WRITE_TAIL;
            end if;
                
        when HARD_REFRESH =>
            end_reconfig <= '1';
            refresh_hard <= '1';
            nextstate <= SAFETY_WINDOW;
					 
        when SAFETY_WINDOW =>
            enable_safety_w <= '1';
            enable_hard <= '0';
            if(sync = '1') then
                nextstate <= HARD_INIT;
            else
                nextstate <= SAFETY_WINDOW;
            end if;    
                
        when HARD_INIT =>
            clear_hard <= '1';
            clear_input_reg <= '1';
            enable_hard <= '0';
            nextstate <= SAFETY_WINDOW2;  

        when SAFETY_WINDOW2 =>
            enable_safety_w <= '1';
            enable_hard <= '0';
            if(sync = '1') then
                nextstate <= INIT;
            else
                nextstate <= SAFETY_WINDOW2;
            end if; 
                
        when others => nextstate <= IDLE;
				
        end case;
    end process;
    
 
    error_reg_process : process(clk, rst, error_reg_clear, error_reg_enable)
    begin

        if(rst = '1') or (error_reg_clear = '1') then  
            error_reg <= '0';
        else  
            if(rising_edge(clk)) then
                if(error_reg_enable = '1') then
                    error_reg <= '1';
                end if;
            end if;
        end if;
    end process;

 
    byte_counter : process(clk, rst, byte_cnt_clear, byte_cnt_enable)
    begin
        if(rst = '1') or (byte_cnt_clear = '1') then  
            byte_cnt <= (others => '0');
        else  
            if(rising_edge(clk)) then
                if(byte_cnt_enable = '1') then
                    byte_cnt <= byte_cnt + 1;
                end if;
            end if;
        end if;
    end process;
    
    
    frame_counter : process(clk, rst, frame_cnt_clear, frame_cnt_enable)
    begin
        if(rst = '1') or (frame_cnt_clear = '1') then  
            frame_cnt <= (others => '0');
        else
            if(rising_edge(clk)) then
                if(frame_cnt_enable = '1') then
                    frame_cnt <= frame_cnt + 1;
                end if;
             end if;
        end if;		  
    end process;
    

    byte_count_register : process(clk, rst)
    begin 
        if(rst = '1') then
            frame_byte_count <= (others => '0');
        elsif rising_edge(clk) then
            if(enable_byte1 = '1') then
                frame_byte_count(13 downto 10) <= data(3 downto 0);
            elsif(enable_byte2 = '1') then
                frame_byte_count(9 downto 2) <= data(7 downto 0);
            end if;    
        end if;
    end process;

end behav;

